<?php
/**
 * copyright (C) 2008 GWE Systems Ltd - All rights reserved
 */

defined( 'JPATH_BASE' ) or die( 'Direct Access to this location is not allowed.' );
use Joomla\Registry\Registry;
use Joomla\CMS\MVC\Controller\FormController;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Filter\InputFilter;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Session\Session;
use Joomla\CMS\Factory;
use Joomla\CMS\Input\Input;
use Joomla\Utilities\ArrayHelper;

jimport('joomla.application.component.controller');
include_once(JPATH_ADMINISTRATOR."/components/com_jevents/jevents.defines.php");

class AdminLocationsController extends FormController {

	function __construct($config = array())
	{
		parent::__construct($config);
		$this->registerTask( 'list',  'overview' );
		$this->registerTask( 'apply',  'apply' );
		$this->registerDefaultTask("list");
		PluginHelper::importPlugin('jevents');
	}

	function overview( )
	{
		$this->_authoriseAccess();

		// get the view
		$viewName = "locations";
		$this->view = $this->getView($viewName,"html");

		// Set the layout
		$this->view->setLayout('overview');
		$this->view->title = Text::_("COM_JEVLOCATIONS_LOCATIONS_LIST");
		$jevuser = JEVHelper::getAuthorisedUser();
		$this->view->jevuser = $jevuser;

		// Get/Create the model
		if ($model =  $this->getModel($viewName, "LocationsModel")) {
			// Push the model into the view (as default)
			$this->view->setModel($model, true);
		}

		$this->view->display();
	}

	/**
	 * Check in of one or more records.
	 *
	 * @return  boolean  True on success
	 *
	 * @since   1.6
	 */
	public function checkin()
	{

	    		// Check for request forgeries.
		Session::checkToken() or jexit(Text::_('JINVALID_TOKEN'));
		$ids = $this->input->post->get('loc_id', array(), 'array');
		$model = $this->getModel('locations', "LocationsModel");
		//var_dump($ids);die;
		$return = $model->checkin($ids);

		if ($return === false)
		{
			// Checkin failed.
			$message = Text::sprintf('JLIB_APPLICATION_ERROR_CHECKIN_FAILED', $model->getError());
			$this->setRedirect(Route::_('index.php?option=com_jevlocations&task=locations.overview', false), $message, 'error');

			return false;
		}
		else
		{
			// Checkin succeeded.
			$message = Text::plural('COM_JEVLOCATIONS_N_ITEMS_CHECKED_IN', count($ids));
			$this->setRedirect(Route::_('index.php?option=com_jevlocations&task=locations.overview', false), $message);

			return true;
		}
	}

	function select( )
	{
		//$this->_authoriseAccess();

		// get the view
		$viewName = "locations";
		$this->view = $this->getView($viewName,"html");

		// Set the layout
		$this->view->setLayout('select');
		$this->view->title = Text::_("COM_JEVLOCATIONS_LOCATIONS_LIST");

		$this->fixCreationPermissions();
		$jevuser = JEVHelper::getAuthorisedUser();
		$this->view->jevuser = $jevuser;

		Factory::getApplication()->input->post->set('filter_state','P');
		// Get/Create the model
		if ($model =  $this->getModel($viewName, "LocationsModel")) {
			// Push the model into the view (as default)
			$this->view->setModel($model, true);
		}

		$this->view->display();
	}

	function edit($key = null, $urlVar = null)
	{
		$jinput = Factory::getApplication()->input;
		$jpost = $jinput->post;
		$cid = $jpost->get( 'cid', $jpost->get('loc_id', array(0), 'array' ) , 'array');
       if (is_array($cid) && isset($cid[0])){
           $cid = (int) $cid[0];
       }

		$this->_authoriseAccess($cid);

		// get the view
		$viewName = "locations";
		$this->view = $this->getView($viewName,"html");

		$jinput->set('hidemainmenu', 1 );

		$returntask	= $jinput->get( 'returntask', "locations.overview");

		if ($returntask!="locations.list" && $returntask!="locations.overview" && $returntask!="locations.select"){
			$returntask="locations.overview";
		}
		$this->view->returntask =  $returntask;

		// Set the layout
		$this->view->setLayout('edit');
		$this->view->title =  Text::_("COM_JEVLOCATIONS_LOCATION_EDIT");
		$jevuser = JEVHelper::getAuthorisedUser();
		$this->view->jevuser = $jevuser;

		// Get/Create the model
		if ($model =  $this->getModel("location", "LocationsModel")) {
			// Push the model into the view (as default)
			$this->view->setModel($model, true);
		}

		// Get the media component configuration settings
		$params = ComponentHelper::getParams('com_media');
		// Set the path definitions
		if (!defined( 'JEVP_MEDIA_BASE' ))
		{
			define('JEVP_MEDIA_BASE', JPATH_ROOT . '/' . $params->get('image_path', 'images/stories'));
			define('JEVP_MEDIA_BASEURL', JURI::root(true) . '/' . $params->get('image_path', 'images/stories'));
		}
		$this->view->display();
	}

	function apply()
	{
        $jinput = Factory::getApplication()->input;
		// Check for request forgeries
		Session::checkToken() or jexit( 'Invalid Token' );

        $cid = $jinput->post->get('cid', array());

        if(is_array($cid) && $cid[0] === '') {
            $cid[0] = 0;
        }

		if($this->dosave($cid))
		{
			$user = Factory::getUser();

			$returntask = "locations.edit";

			if ($user->id !=0  && method_exists($this,str_replace("locations.","",$returntask))){
				$returntask = str_replace("locations.","",$returntask);
				//doSave function handles $cid within it's self, so we need to handle it here for new locations on save too.
				if(is_array($cid)) {
				    $cid = (int) $cid[0];
				}
                if ($cid === 0){
                    // get last saved location id from registry
                    $reg = JevRegistry::getinstance("com_jevlocations");
                    $cid = $reg->get("last_loc_id", 0);
                    Factory::getApplication()->input->set( 'cid', $cid);
                }

				return $this->$returntask();
			}

			$tmpl = "";
			if ($returntask == "locations.select" || Factory::getApplication()->input->getString("tmpl","") == "component"){
				$tmpl = "&tmpl=component";
			}

			// anon users only
			$Itemid = Factory::getApplication()->getMenu()->getActive()->id; // RSH 10/11/10 Make 1.5/1.6 compatible
			if ($user->id ==0) {
				die('2');
				$link = Route::_('index.php?option=com_jevlocations&task='.$returntask. $tmpl."&Itemid=".$Itemid	);
			}
			else {
				die('3');
				$link = Route::_('index.php?option=com_jevlocations&task='.$returntask. $tmpl."&Itemid=".$Itemid	);
			}
			$this->setRedirect($link, $msg); //$msg not set...
		}
	}

	function batch($model = null){
		Session::checkToken() or jexit(Text::_('JINVALID_TOKEN'));

		// Set the model
		$model = $this->getModel('locations', '', array());

		// Preset the redirect
		$this->setRedirect(Route::_('index.php?option=com_jevlocations&task=locations.overview' . $this->getRedirectToListAppend(), false));

		$vars = $this->input->post->get('batch', array(), 'array');
		$cid  = $this->input->post->get('loc_id', array(), 'array');

		$model->batch($vars, $cid);

		return parent::batch($model);
	}

	function cancel($key = null){
		// Check for request forgeries
		Session::checkToken() or jexit( 'Invalid Token' );

		$cid = Factory::getApplication()->input->post->get( 'cid', Factory::getApplication()->input->post->get( 'loc_id', array(0),'array'), 'array');
		$this->_authoriseAccess((int) $cid[0]);

		$model =  $this->getModel("location", "LocationsModel");

		$model->getData();

		// Check the table in so it can be edited.... we are done with it anyway
		$model->checkin();

		if (Factory::getApplication()->input->getString("tmpl","")=="component" && Factory::getApplication()->input->getString('pop',0)==1) {
			ob_end_clean();
				?>
				<script type="text/javascript">
                    window.parent.closeJevModalBySelector('#jevLocationModal');
				</script>
				<?php
				exit();
		}

		$user = Factory::getUser();
		if ($user->id ==0) {
			$returntask	= Factory::getApplication()->input->getCmd( 'returntask', "locations.locations");
			if ($returntask!="locations.select"){
				Factory::getApplication()->input->set("returntask","locations.locations");
			}

		}

		$returntask	= Factory::getApplication()->input->getCmd( 'returntask', "locations.overview");
		if ($returntask!="locations.list" && $returntask!="locations.overview" && $returntask!="locations.select"  && $returntask!="locations.locations") {
			$returntask="locations.overview";
		}
		$tmpl = "";

		if ($user->id !=0  && method_exists($this,str_replace("locations.","",$returntask))){
			$returntask = str_replace("locations.","",$returntask);
			return $this->$returntask();
		}
		if ($returntask=="locations.select" || Factory::getApplication()->input->getString("tmpl","")=="component"){
			$tmpl ="&tmpl=component";
		}
		$link = Route::_('index.php?option=com_jevlocations&task='.$returntask . $tmpl);
		$this->setRedirect($link);
	}

	function dosave($cid = array(0))
	{
	    $jinput = Factory::getApplication()->input;
		$params = ComponentHelper::getParams("com_jevlocations");
		$post	= $jinput->post->getArray(array(), null, 'RAW');

		if(!$params->get("allowraw", 0))
		{
			if (version_compare(JVERSION, '3.7.1', '>='))
			{
				$filter = InputFilter::getInstance(array(), array(), 1, 1);

				//Joomla! no longer provides HTML allowed in Input so we need to fetch raw
				//Then filter on through with InputFilter to HTML

				foreach ($post as $key => $row)
				{
					//Single row check
					if (!is_array($row))
					{
						$post[$key] = $filter->clean($row, 'HTML');
					}
					else
					{
						//1 Deep row check
						foreach ($post[$key] as $key1 => $sub_row)
						{
							//2 Deep row check
							if (!is_array($sub_row))
							{
								$post[$key][$key1] = $filter->clean($sub_row, 'HTML');
							}
							else
							{
								foreach ($sub_row as $key2 => $sub_sub_row)
								{
									$post[$key][$key1][$key2] = $filter->clean($sub_sub_row, 'HTML');
								}
							}
						}
					}
				}
			}
		}

		if (isset($post['targetmenu']))
		{
			$post['targetmenu'] = (int) $post['targetmenu'];
		}

		$post['loc_id'] = (int) $cid[0];
		$this->_authoriseAccess($post['loc_id']);

		// ensure no dodgy setting of global values !
		$jevuser = JEVHelper::getAuthorisedUser(); // Not being used...


		$user = Factory::getUser();
		if (!JevLocationsHelper::canCreateGlobal())
		{
			$post["global"]=0;
		}
		else if (!isset($post["global"]))
		{
			// anon users only - set global true and published false
			if ($user->id == 0)
			{
				$post["global"]=1;
				$post["published"]=$params->get("anonpublished",0);
				$returntask	= $jinput->get( 'returntask', "locations.locations");
				if ($returntask!="locations.select")
				{
				    $jinput->set("returntask","locations.locations");
				}
			}
		}
		else
		{
			if ($user->id == 0)
			{
				$post["published"]=$params->get("anonpublished",0);
			}
		}

		//We check now max number of own locations
		if( ($post['loc_id'] == 0) && !$post["global"] && !JevLocationsHelper::canCreateOwn())
		{
			return false;
		}

		$model = $this->getModel("location", "LocationsModel");

		if ($model->store($post))
		{
            $msg = Text::_( 'COM_JEVLOCATIONS_LOCATION_SAVED' , true);
			$params = ComponentHelper::getParams("com_jevlocations");
			if ($user->id ==0 && !$params->get("anonpublished",0)) {
				$msg = Text::_( 'COM_JEVLOCATIONS_LOCSAVED_REVIEWED' );
			}
			if ($user->id == 0) {
				$location = $model->lastrow;
				$this->notifyAdmin($location);
			}

			Factory::getApplication()->enqueueMessage($msg, 'message');

			if ($this->task === 'savecopy') {
			    //Set the new loc id for correct redirection.
			    $jinput->set('loc_id', $model->lastrow->loc_id);
			    $jinput->set('cid', array(0 => $model->lastrow->loc_id));
            }

		}
		else
		{
			$msg = Text::_( 'COM_JEVLOCATIONS_ERROR_SAVING_LOCATION', true)." - ". $model->getError() ;
			Factory::getApplication()->enqueueMessage($msg, 'error');
			return false;
		}

		// Check the table in so it can be edited.... we are done with it anyway
		$model->checkin();

		// Are we saving a newly created location?  If so select it using JS then close the modal and exit
		if (Factory::getApplication()->input->getString("tmpl","") == "component" )
		{
			ob_end_clean();
				?>
				<?php

				if (isset($model->lastrow->loc_id) && $model->lastrow->loc_id>0) {

				?>
				<div id="location"><?php echo $model->lastrow->title;?></div>
				 <?php } ?>
				<script type="text/javascript">
					function selectThisLocation(locid, elem){
						var title = elem.innerHTML;
						var locn = window.parent.document.getElementById('locn');
						if (locn){
							locn.value  = locid;
						}
						var evlocation = window.parent.document.getElementById('evlocation');
						if (evlocation){
							evlocation.value = title;
							// If actually selecting a location for a menu item we do something different:
							var menu_location = window.parent.document.getElementById('menu_location');
							if (menu_location){
								menu_location.value = "jevl:"+locid;
							}
						}
						// else this is a menu so do something different
						else {
							return window.parent.sortableLocations.selectThisLocation(locid, elem);
						}

                        window.parent.closeJevModalBySelector('#jevLocationModal');
						return false;
					}

					<?php if (isset($model->lastrow->loc_id) && $model->lastrow->loc_id>0) { ?>
						selectThisLocation(<?php echo $model->lastrow->loc_id;?>, document.getElementById('location'));
					 <?php } ?>

					window.parent.alert("<?php echo $msg;?>");
				</script>
				<?php
				exit();
		}
		return true;
	}

	function save($key = null, $urlVar = null)
	{
		// Check for request forgeries
		Session::checkToken() or jexit( 'Invalid Token' );
		$cid = Factory::getApplication()->input->post->get( 'cid', Factory::getApplication()->input->post->get( 'loc_id', array(0),'array'), 'array');

		if($this->dosave($cid))
		{
			$user = Factory::getUser();

			$returntask	= Factory::getApplication()->input->getCmd( 'returntask', "locations.overview");
			if ($returntask!="locations.list" && $returntask!="locations.locations" && $returntask!="locations.overview" && $returntask!="locations.select"){
				$returntask="locations.overview";
			}
			if ($user->id !=0  && method_exists($this,str_replace("locations.","",$returntask))){
				$returntask = str_replace("locations.","",$returntask);
				return $this->$returntask();
			}

			$tmpl = "";
			if ($returntask=="locations.select" || Factory::getApplication()->input->getString("tmpl","")=="component"){
				$tmpl ="&tmpl=component";
			}

			// anon users only
			$Itemid = Factory::getApplication()->getMenu()->getActive()->id; // RSH 10/11/10 Make 1.5/1.6 compatible
			if ($user->id ==0) {
				$link = Route::_('index.php?option=com_jevlocations&task=' . $returntask . $tmpl . "&Itemid=" . $Itemid	);
			}
			else {
				$link = Route::_('index.php?option=com_jevlocations&task=' . $returntask . $tmpl . "&Itemid=" . $Itemid	);
			}
			$this->setRedirect($link, ""); // "" not set.
		}
	}

	function savecopy()
	{
		// Check for request forgeries
        $jinput = Factory::getApplication()->input;

		Session::checkToken() or jexit( 'Invalid Token' );

		$save = $this->dosave();

		$user = Factory::getUser();

		$returntask	= "locations.edit";
		$Itemid = Factory::getApplication()->getMenu()->getActive()->id; // RSH 10/11/10 Make 1.5/1.6 compatible
        $itemid_string = isset($Itemid) && $Itemid !== '' ? '&Itemid=' . $Itemid : '';
		$tmpl = "";
		if ($returntask=="locations.select" || Factory::getApplication()->input->getString("tmpl","")=="component"){
			$tmpl ="&tmpl=component";
		}

		if($jinput->get('task', '') === 'locations.savecopy') {
			$cid = $jinput->get('loc_id');
			$link = 'index.php?option=com_jevlocations&task='.$returntask. '&cid[]=' . $cid . $tmpl. $itemid_string;
			$this->setRedirect($link, ""); // &msg not defined.
            return true;
		}

		if ($user->id != 0  && method_exists($this,str_replace("locations.","",$returntask))){
			$returntask = str_replace("locations.","",$returntask);
			return $this->$returntask();
		}

		// anon users only
		if ($user->id ==0) {
			$link = Route::_('index.php?option=com_jevlocations&task='.$returntask. $tmpl."&Itemid=".$Itemid	);
		}
		else {
			$link = Route::_('index.php?option=com_jevlocations&task='.$returntask. $tmpl."&Itemid=".$Itemid	);
		}
		$this->setRedirect($link, ""); // &msg not defined.
	}


	function delete()
	{
		// Check for request forgeries
		Session::checkToken() or jexit( 'Invalid Token' );

		$cid = Factory::getApplication()->input->post->get( 'cid', Factory::getApplication()->input->post->get( 'loc_id', array(),'array'), 'array');
		$cid = ArrayHelper::toInteger($cid);

		$this->_authoriseAccess($cid);

		if (count( $cid ) < 1) {
			throw new Exception( Text::_( 'COM_JEVLOCATIONS_SELECT_ITEM_TO_DELETE' ) , 500);

		}

		$model =  $this->getModel("location", "LocationsModel");
		$model->delete($cid);

		$returntask	= Factory::getApplication()->input->getCmd( 'returntask', "locations.overview");
		if ($returntask!="locations.list" && $returntask!="locations.overview" && $returntask!="locations.select"){
			$returntask="locations.overview";
		}
		if (method_exists($this,str_replace("locations.","",$returntask))){
			$returntask = str_replace("locations.","",$returntask);
			return $this->$returntask();
		}

		$tmpl = "";
		if (Factory::getApplication()->input->getString("tmpl","")=="component"){
			$tmpl ="&tmpl=component";
		}

		$this->setRedirect( Route::_('index.php?option=com_jevlocations&task=locations.overview' . $tmpl));
	}


	function publish()
	{
		// Check for request forgeries
		Session::checkToken() or jexit( 'Invalid Token' );

		$cid = Factory::getApplication()->input->post->get( 'cid', Factory::getApplication()->input->post->get( 'loc_id', array(),'array'), 'array');
		$cid = ArrayHelper::toInteger($cid);

		$this->_authoriseAccess($cid);

		if (count( $cid ) < 1) {
			throw new Exception( Text::_( 'COM_JEVLOCATIONS_SELECT_ITEM_TO_PUBLISH' ) , 500);
		}

		$model =  $this->getModel("location", "LocationsModel");
		if(!$model->publish($cid, 1)) {
			echo "<script> alert('".$model->getError(true)."'); window.history.go(-1); </script>\n";
		}

		$returntask	= Factory::getApplication()->input->getCmd( 'returntask', "locations.overview");
		if ($returntask!="locations.list" && $returntask!="locations.overview" && $returntask!="locations.select"){
			$returntask="locations.overview";
		}
		if (method_exists($this,str_replace("locations.","",$returntask))){
			$returntask = str_replace("locations.","",$returntask);
			return $this->$returntask();
		}

		$tmpl = "";
		if ( Factory::getApplication()->input->getString("tmpl","")=="component"){
			$tmpl ="&tmpl=component";
		}

		$this->setRedirect(Route::_( 'index.php?option=com_jevlocations&task=locations.overview' . $tmpl));
	}


	function unpublish()
	{
		// Check for request forgeries
		Session::checkToken() or jexit( 'Invalid Token' );

		$cid = Factory::getApplication()->input->post->get( 'cid', Factory::getApplication()->input->post->get( 'loc_id', array(),'array'), 'array');
		$cid = ArrayHelper::toInteger($cid);

		$this->_authoriseAccess($cid);

		if (count( $cid ) < 1) {
			throw new Exception( Text::_( 'COM_JEVLOCATIONS_SELECT_ITEM_TO_UNPUBLISH' ) , 500);
		}

		$model =  $this->getModel("location", "LocationsModel");
		if(!$model->publish($cid, 0)) {
			echo "<script> alert('".$model->getError(true)."'); window.history.go(-1); </script>\n";
		}

		$returntask	= Factory::getApplication()->input->getCmd( 'returntask', "locations.overview");
		if ($returntask!="locations.list" && $returntask!="locations.overview" && $returntask!="locations.select"){
			$returntask="locations.overview";
		}
		if (method_exists($this,str_replace("locations.","",$returntask))){
			$returntask = str_replace("locations.","",$returntask);
			return $this->$returntask();
		}

		$tmpl = "";
		if (Factory::getApplication()->input->getString("tmpl","")=="component"){
			$tmpl ="&tmpl=component";
		}

		$this->list();
		//$this->setRedirect( Route::_('index.php?option=com_jevlocations&task=locations.overview'.$tmpl) );
	}


	function globalise()
	{
		// Check for request forgeries
		Session::checkToken() or jexit( 'Invalid Token' );
		if (JevLocationsHelper::canCreateGlobal()){

		$cid = Factory::getApplication()->input->post->get( 'cid', Factory::getApplication()->input->post->get( 'loc_id', array(),'array'), 'array');
			$cid = ArrayHelper::toInteger($cid);

			$this->_authoriseAccess($cid);

			if (count( $cid ) < 1) {
				throw new Exception( Text::_( 'COM_JEVLOCATIONS_SELECT_ITEM_TO_PUBLISH' ) , 500);
			}

			$model =  $this->getModel("location", "LocationsModel");
			if(!$model->globalise($cid, 1)) {
				echo "<script> alert('".$model->getError(true)."'); window.history.go(-1); </script>\n";
			}

			$returntask	= Factory::getApplication()->input->getCmd( 'returntask', "locations.overview");
			if ($returntask!="locations.list" && $returntask!="locations.overview" && $returntask!="locations.select"){
				$returntask="locations.overview";
			}
			if (method_exists($this,str_replace("locations.","",$returntask))){
				$returntask = str_replace("locations.","",$returntask);
				return $this->$returntask();
			}

			$tmpl = "";
			if ( Factory::getApplication()->input->getString("tmpl","")=="component"){
				$tmpl ="&tmpl=component";
			}
		}
		$this->setRedirect(Route::_( 'index.php?option=com_jevlocations&task=locations.overview' . $tmpl));
	}


	function privatise()
	{
		// Check for request forgeries
		Session::checkToken() or jexit( 'Invalid Token' );

		if (JevLocationsHelper::canCreateGlobal()){

		$cid = Factory::getApplication()->input->post->get( 'cid', Factory::getApplication()->input->post->get( 'loc_id', array(),'array'), 'array');
			$cid = ArrayHelper::toInteger($cid);

			$this->_authoriseAccess($cid);

			if (count( $cid ) < 1) {
				throw new Exception(500, Text::_( 'COM_JEVLOCATIONS_SELECT_ITEM_TO_UNPUBLISH' ) );
			}

			$model =  $this->getModel("location", "LocationsModel");
			if(!$model->globalise($cid, 0)) {
				echo "<script> alert('".$model->getError(true)."'); window.history.go(-1); </script>\n";
			}

			$returntask	= Factory::getApplication()->input->getCmd( 'returntask', "locations.overview");
			if ($returntask!="locations.list" && $returntask!="locations.overview" && $returntask!="locations.select"){
				$returntask="locations.overview";
			}
			if (method_exists($this,str_replace("locations.","",$returntask))){
				$returntask = str_replace("locations.","",$returntask);
				return $this->$returntask();
			}

			$tmpl = "";
			if (Factory::getApplication()->input->getString("tmpl","")=="component"){
				$tmpl ="&tmpl=component";
			}

		}
		//$this->list();
		$this->setRedirect( Route::_('index.php?option=com_jevlocations&task=locations.overview'.$tmpl) );
	}

	function upload(){

		// Check for request forgeries
		Session::checkToken() or jexit( 'Invalid Token' );

		$this->view = $this->getView("locations","html");

		if (!JevLocationsHelper::canUploadImages()){
				$this->view->setLayout('noauth');
				$this->view->msg = Text::_("COM_JEVLOCATIONS_NOT_AUTHORISED");
				$this->view->display();
				return;
		}

		$this->view->setLayout('upload');

		$folder		= Factory::getApplication()->input->getPath( 'folder', '');
		$field		= Factory::getApplication()->input->getString( 'field', '', '' );

		// Get the media component configuration settings
		$params = ComponentHelper::getParams('com_media');
		// Set the path definitions
		if (!defined( 'JEVP_MEDIA_BASE' ))
		{
			define('JEVP_MEDIA_BASE', JPATH_ROOT . '/' . $params->get('image_path', 'images/stories'));
			define('JEVP_MEDIA_BASEURL', JURI::root(true) . '/' . $params->get('image_path', 'images/stories'));
		}

		$jevLocationsHelper = new JevLocationsHelper();
		foreach ($_FILES as $fname=>$file) {
			if ($fname != $field."_file") continue;
			if (strpos($fname,"image")===0){
				$filename = $jevLocationsHelper->processImageUpload($fname);
				$this->view->filetype = "image";
				$oname = $_FILES[$fname]['name'];
				$this->view->oname = $oname;
			}
		}
		$this->view->fname = $field."_file";
		$this->view->filename = $filename;
		$this->view->display();

	}


	function orderup()
	{
		$this->_authoriseAccess();

		// Check for request forgeries
		Session::checkToken() or jexit( 'Invalid Token' );

		$model =  $this->getModel("locations", "LocationsModel");
		$model->move(-1);

		$tmpl = "";
		if (Factory::getApplication()->input->getString("tmpl","")=="component"){
			$tmpl ="&tmpl=component";
		}

		$this->setRedirect( Route::_('index.php?option=com_jevlocations&task=locations.overview' .$tmpl));
	}

	function orderdown()
	{
		$this->_authoriseAccess();

		// Check for request forgeries
		Session::checkToken() or jexit( 'Invalid Token' );

		$model =  $this->getModel("locations", "LocationsModel");
		$model->move(1);
		$tmpl = "";
		if (Factory::getApplication()->input->getString("tmpl","")=="component"){
			$tmpl ="&tmpl=component";
		}

		$this->setRedirect( Route::_('index.php?option=com_jevlocations&task=locations.overview'.$tmpl) );
	}

	function saveorder()
	{
		// Check for request forgeries
		Session::checkToken() or jexit( 'Invalid Token' );

		$cid = Factory::getApplication()->input->post->get( 'cid', Factory::getApplication()->input->post->get( 'loc_id', array(),'array'), 'array');
		$order 	= Factory::getApplication()->input->post->get( 'order', array(), 'array' );
		$cid = ArrayHelper::toInteger($cid);
		$order = ArrayHelper::toInteger($order);

		$this->_authoriseAccess($cid);

		$model =  $this->getModel("location", "LocationsModel");
		$model->saveorder($cid, $order);

		$tmpl = "";
		if (Factory::getApplication()->input->getString("tmpl","")=="component"){
			$tmpl ="&tmpl=component";
		}

		$msg = 'New ordering saved';
		$this->setRedirect( Route::_('index.php?option=com_jevlocations&task=locations.overview' .  $tmpl, false),$msg);
	}


	/**
	 * This mechanism currently only checks to see if user is authorised to do anything to locations
	 *
	 * @param unknown_type $locid
	 */
	function _authoriseAccess($locid=0)
	{
		$jeventsParams = ComponentHelper::getParams('com_jevents');
		$params = ComponentHelper::getParams("com_jevlocations");

		$authorisedOnly = $jeventsParams->get("authorisedonly",0);

		$jevuser = JEVHelper::getAuthorisedUser();

		if (is_null($jevuser))
		{
			if (!$authorisedOnly)
			{
				$juser = Factory::getUser();

				if( $juser->authorise('core.create', 'com_jevlocations'))
				{
					return true;
				}
			}

			// check if anonymous users can add or save locations
			if (!isset($this->_task))
			{
				$this->_task = $this->task;
			}

			if (($locid==0 && $this->_task=='save') || (is_array($locid) && isset($locid[0]) && $locid[0]==0 && $this->_task=='edit' ) || $this->_task=='cancel')
			{
				$anoncreate = $params->get("anoncreate",25);
				$user = Factory::getUser();
				if ($anoncreate && $user->id==0)
				{
					// Now make sure captcha is valid
					if ($this->_task=='save')
					{
						$plugin = PluginHelper::getPlugin('jevents', 'jevanonuser' );
						$pluginparams = new Registry($plugin->params);

						if ($pluginparams->get("recaptchapublic",false))
						{
							$mainframe = Factory::getApplication();  // RSH 10/11/10 - make 1.5/1.6 compatible
							$jevrecaptcha = $mainframe->getUserState("jevrecaptcha");

							if ($jevrecaptcha == "ok")
							{
								$mainframe->setUserState("jevrecaptcha","error");
								return true;
							}
							// Belt and braces
							if (version_compare(JVERSION, "1.6.0", 'ge'))
							{
								require_once(JPATH_SITE.'/plugins/jevents/jevanonuser/anonuserlib/recaptcha.php');
							}
							else
							{
								require_once(JPATH_SITE.'/plugins/jevents/anonuserlib/recaptcha.php');
							}

							$response = recaptcha_check_answer($pluginparams->get("recaptchaprivate",false),Factory::getApplication()->input->getString("REMOTE_ADDR","","server"), Factory::getApplication()->input->getString("recaptcha_challenge_field"),Factory::getApplication()->input->getString("recaptcha_response_field"));

							if ($response->is_valid)
							{
								// check for valid email and name ?
								$mainframe->setUserState("jevrecaptcha","ok");
								return true;
							}
							else
							{
								Factory::getLanguage()->load( 'plg_jevents_jevanonuser',JPATH_ADMINISTRATOR );
								echo "<script> alert('".Text::_("COM_JEVLOCATIONS_RECAPTCHA_ERROR",true)."'); window.history.go(-1); </script>\n";
								exit();
							}
						}
					}
					else
					{
						return true;
					}
				}
				else if ($jevuser->cancreateown || $jevuser->cancreateglobal)
				{
					// if jevents is not in authorised only mode then switch off this user's permissions
					if (!$authorisedOnly)
					{
						$maxOwnLocations = $params->get("max_art",5);
						$juser = Factory::getUser();

						if( !$juser->authorise('core.create', 'com_jevlocations'))
						{
								$jevuser->cancreateown=false;
								$jevuser->cancreateglobal=false;
						}

					}

					return true;
				}

		$isLocationCreator = false;

		Factory::getApplication()->triggerEvent('isLocationCreator', array(& $isLocationCreator));
		if ($isLocationCreator)
		{
			return $isLocationCreator;
		}

		$this->setRedirect( 'index.php', Text::_("COM_JEVLOCATIONS_NOT_AUTHORISED") );
		$this->redirect();
			}
		}
	}

	function fixCreationPermissions()
	{
		$jevuser = JEVHelper::getAuthorisedUser();

		if 	($jevuser && ($jevuser->cancreateown || $jevuser->cancreateglobal))
		{
			// if jevents is not in authorised only mode then switch off this user's permissions
			$params = ComponentHelper::getParams('com_jevents');
			$authorisedonly = $params->get("authorisedonly",0);

			if (!$authorisedonly)
			{
				$params = ComponentHelper::getParams("com_jevlocations");
				$loc_own = $params->get("loc_own",25);
				$juser = Factory::getUser();

				if (!$juser->authorise('core.create', 'com_jevlocations'))
				{
					$jevuser->cancreateown=false;
					$jevuser->cancreateglobal=false;
				}

				$loc_global = $params->get("loc_global",24);

				if (version_compare(JVERSION, "1.6.0", 'ge'))
				{
					if( !$juser->authorise('core.createglobal', 'com_jevlocations'))
					{
						$jevuser->cancreateown=false;
						$jevuser->cancreateglobal=false;
					}
				}
				else if ($juser->gid < intval($loc_global))
				{
					$jevuser->cancreateown=false;
					$jevuser->cancreateglobal=false;
				}
			}

			return true;
		}
	}


	private function notifyAdmin($location){
		$params = ComponentHelper::getParams("com_jevlocations");
		$anoncreate = $params->get("anoncreate",25);
		$user = Factory::getUser();
		if ($anoncreate && $user->id==0){
			$locadmin = $params->get("locadmin",62);

			$subject = $params->get("notifysubject","");
			$message = $params->get("notifymessage","");
			if ($subject =="" || $message =="") return;
			$message = str_replace("{TITLE}",$location->title,$message);
			$message = str_replace("{NAME}",$location->anonname,$message);
			$message = str_replace("{EMAIL}",$location->anonemail,$message);
			$adminuser = JEVHelper::getUser($locadmin);
			Factory::getMailer()->sendMail($adminuser->email,$adminuser->name,$adminuser->email,$subject,$message);
		}
	}
}
