<?php

use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Filesystem\File;

/**
 * copyright (C) 2008 GWE Systems Ltd - All rights reserved
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC' ) or die();

jimport( 'joomla.application.component.view');

/**
 * HTML View class for the component
 *
 * @static
 */

class AdminCpanelViewCpanel extends JevlocationsAbstractView
{
	/**
	 * Control Panel display function
	 *
	 * @param template $tpl
	 */
	function cpanel($tpl = null)
	{
		jimport('joomla.html.pane');
		HTMLHelper::_('bootstrap.framework');

		HTMLHelper::stylesheet('com_jevlocations/jevlocations3x.css',array(),true);
		JHTML::stylesheet('administrator/components/com_jevlocations/assets/css/eventsadmin.css');
		// Lets check if we have editted before! if not... rename the custom file.
		if (File::exists(JPATH_SITE . "/components/com_jevents/assets/css/jevcustom.css"))
		{
			// It is definitely now created, lets load it!
			JEVHelper::stylesheet('jevcustom.css', 'components/' . JEV_COM_COMPONENT . '/assets/css/');
		}

		//We check Google Maps Key
		$params = ComponentHelper::getParams('com_jevlocations');
		if ($params->get('mapsprovider','googlemaps') == "googlemaps")
		{
			if (!JevLocationsHelper::getApiKey())
			{
			    $msg = Text::sprintf('COM_JEVLOCATIONS_GOOGLEMAPSKEY_WARNING', 'https://www.jevents.net/blog/changes-in-google-maps-terms-of-service');
				$msg .= "<br><br>" . Text::_('COM_JEVLOCATIONS_GOOGLEMAPSKEY_ALTERNATIVE');
				Factory::getApplication()->enqueueMessage( $msg, 'warning');
			}

			if (JevLocationsHelper::getParameter('googlemaps-service', "googlestaticmapskey", ""))
			{
				if (!JevLocationsHelper::getApiKey())
				{
					Factory::getApplication()->enqueueMessage(Text::sprintf('COM_JEVLOCATIONS_GOOGLESTATICMAPSKEY_NOGOOGLEMAPS_WARNING', 'https://www.jevents.net/blog/changes-in-google-maps-terms-of-service'), 'warning');
				}
				else if (JevLocationsHelper::getParameter('googlemaps-service', "googlestaticmapskey", "") !== JevLocationsHelper::getApiKey())
				{
					Factory::getApplication()->enqueueMessage(Text::sprintf('COM_JEVLOCATIONS_GOOGLESTATICMAPSKEY_WITH_DIFFERENT_GOOGLEMAPS_WARNING', 'https://www.jevents.net/blog/changes-in-google-maps-terms-of-service'), 'warning');
				}
			}
		}


		$document = Factory::getDocument();
		$document->setTitle(Text::_('JEVENTS_JEVENTS_LOCATION_MANAGER') . ' :: ' .Text::_( 'CONTROL_PANEL' ));

		// Set toolbar items for the page
        JToolBarHelper::title(Text::_('JEVENTS_JEVENTS_LOCATION_MANAGER') . ' :: ' . Text::_('CONTROL_PANEL'), 'jevlocations');

        if (GSLMSIE10)
        {
	        JToolBarHelper::preferences('com_jevlocations', '580', '750');  // RSH 10/12/10 Use default width of '800' to accomodate permissions

	        JevLocationsHelper::addSubmenu();
	        $this->sidebar = JHtmlSidebar::render();
        }


		// Summary Data
		$db = Factory::getDbo();

		$query = $db->getQuery(true);
		$query->select ("count(loc_id) ")
			->from('#__jev_locations')
			->where('published = 1');
		$db->setQuery($query);
		$this->totalvenues = (int) $db->loadResult();

		$query = $db->getQuery(true);
		$query->select ("count(id) ")
			->from('#__jevlocation_categories')
			->where('published = 1');
		$db->setQuery($query);
		$this->totalcsc = (int) $db->loadResult();

		$query = $db->getQuery(true);
		$query->select ("count(id) ")
			->from('#__categories')
			->where('published = 1')
			->where('extension = "com_jevlocations"');
		$db->setQuery($query);
		$this->totalcategories = (int) $db->loadResult();

	}


	 /**
	 * This method creates a standard cpanel button
	 *
	 * @param string $link
	 * @param string $image
	 * @param string $text
	 * @param string $path
	 * @param string $target
	 * @param string $onclick
	 * @access protected
	 */
	function _quickiconButton( $link, $image, $text, $path=null, $target='', $onclick='' ) {
	 	if( $target != '' ) {
	 		$target = 'target="' .$target. '"';
	 	}
	 	if( $onclick != '' ) {
	 		$onclick = 'onclick="' .$onclick. '"';
	 	}
	 	if( $path === null || $path === '' ) {
	 		$option=Factory::getApplication()->input->getCmd("option");
	 		$path = 'administrator/components/'.$option.'/assets/images/';
	 	}
		$alttext = str_replace("<br/>", " ", $text);
		?>
		<div style="float:left;">
			<div class="icon">
				<a href="<?php echo $link; ?>" <?php echo $target;?>  <?php echo $onclick;?> title="<?php echo $alttext;?>">
					<?php
					//echo JHTML::_('image.administrator', $image, $path, NULL, NULL, $text );
					if (strpos($path, '/')===0){
						$path = substr($path,1);
					}
					echo JHTML::_('image', $path.$image, $alttext , array('title'=>$alttext), false);
					//HTMLHelper::_('image', 'mod_languages/'.$menuType->image.'.gif', $alt, array('title'=>$menuType->title_native), true)
					?>
					<span><?php echo $text; ?></span>
				</a>
			</div>
		</div>
		<?php
	}

	function _quickiconButtonWHover($link, $image, $image_hover, $text, $path = '/administrator/images/', $target = '', $onclick = '')
	{
		if ($target != '')
		{
			$target = 'target="' . $target . '"';
		}
		if ($onclick != '')
		{
			$onclick = 'onclick="' . $onclick . '"';
		}
		if ($path === null || $path === '')
		{
			$path = '/administrator/images/';
		}
		$alttext = str_replace("<br/>", " ", $text);
		?>
		<div id="cp_icon_container">
			<div class="cp_icon">
				<a href="<?php echo $link; ?>" <?php echo $target; ?>  <?php echo $onclick; ?> title="<?php echo $alttext; ?>">
					<?php
					//echo JHTML::_('image.administrator', $image, $path, NULL, NULL, $text );
					if (strpos($path, '/') === 0)
					{
						$path = substr($path, 1);
					}
					$atributes = array('title' => $alttext, 'onmouseover' => 'this.src=\'../' . $path . $image_hover . '\'', 'onmouseout' => 'this.src=\'../' . $path . $image . '\'' );

					echo JHTML::_('image', $path . $image, $alttext, $atributes, false);
					//HTMLHelper::_('image', 'mod_languages/'.$menuType->image.'.gif', $alt, array('title'=>$menuType->title_native), true)
					?>
					<span><?php echo $text; ?></span>
				</a>
			</div>
		</div>
	<?php

	}

	/**
	 * Routine to hide submenu suing CSS since there are no paramaters for doing so without hiding the main menu
	 *
	 */
	function _hideSubmenu(){
		$option = JEVEX_COM_COMPONENT;
		JHTML::stylesheet(  'administrator/components/'.$option.'/assets/css/hidesubmenu.css' );
	}

}
