<?php

use Joomla\Registry\Registry;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;

/**
 * JEvents Component for Joomla 1.5.x
 *
 * @version $Id$
 * @package JEvents
 * @copyright Copyright (C) 2008-2009 GWE Systems Ltd
 * @license GNU/GPLv2, see http://www.gnu.org/licenses/gpl-2.0.html
 * @link http://www.jevents.net
 */

defined('_JEXEC' ) or die( 'No Direct Access' );

// searches location of event
class jevBrowserLocationfilterFilter extends jevFilter
{
	const filterType="browserlocationsearch";

	function __construct($tablename, $filterfield, $isstring=true){
		$this->filterLabel=Text::_( 'JEV_SEARCH_NEAR_LOCATION' );
		$this->filterNullValue="";
		$this->filterType=self::filterType;

		parent::__construct($tablename,$filterfield, true);

		$lang 		= Factory::getLanguage();
		$lang->load("plg_jevents_jevlocations", JPATH_ADMINISTRATOR);	// RSH 11/2/10 - Make sure language file is loaded

		// Should these be ignored?
		$reg = Factory::getConfig();
		$modparams = $reg->get("jev.modparams",false);
		if ($modparams && $modparams->get("ignorefiltermodule",false)){
			$this->filter_value = Factory::getApplication()->input->getString($this->filterType.'_fv', $this->filterNullValue );
		}

		if (intval(Factory::getApplication()->input->getInt('filter_reset',0))){
			$this->filter_value = "";
		}

		$plugin = PluginHelper::getPlugin("jevents","jevlocations");
		if ($plugin){
			$this->params = new Registry($plugin->params);
		}
		else  {
			$this->params = false;
		}
		$compparams = ComponentHelper::getParams("com_jevlocations");
		if ($compparams->get("gwidth",-1) !=-1){
			$this->params = $compparams;
		}

	}

	function _createFilter($prefix=""){
		if (!$this->params) return "";
		if (Factory::getApplication()->input->getInt("skipfilter",0)) return "";
		if (!$this->filterField ) return "";
		if (trim($this->filter_value)==$this->filterNullValue || strlen(trim($this->filter_value))<3) return "";
		if (strpos($this->filter_value,",")<=0) return "";

		$googlesearchdistance= $this->params->get("geosearchdistance",10);

		// convert to kilometers
		if ($this->params->get("scale","miles")=="miles"){
			$googlesearchdistance *= 1.609344;
		}

		list($lat,$lon) = explode(",",$this->filter_value);
		$lon = deg2rad($lon);
		$lat = deg2rad($lat);

		// see http://www.movable-type.co.uk/scripts/latlong.html
		// http://www.artfulsoftware.com/infotree/queries.php?&bw=1280
		$db = Factory::getDBO();
		//$filter = "ACOS(SIN(loc.geolat*PI()/180)*SIN($lat)+COS(loc.geolat*PI()/180)*COS($lat)*COS($lon-(loc.geolon*PI()/180)))*6371 < $googlesearchdistance";
		$filter = "ACOS(SIN(loc.geolat*PI()/180)*SIN($lat)+COS(loc.geolat*PI()/180)*COS($lat)*COS($lon-(loc.geolon*PI()/180)))*6371 < $googlesearchdistance";

		return $filter;
	}

	function _createJoinFilter($prefix=""){
		if (!$this->params) return "";
		// location looup is always included so no need for this!
		//return " #__jev_locations as loc ON loc.loc_id=det.loc_id";
		return "";
	}

	function _createfilterHTML(){
		$filterList=array();
		if (!$this->params) return $filterList;
		if (!$this->filterField) return $filterList;

		$db = Factory::getDBO();

		$compparams = ComponentHelper::getParams("com_jevlocations");
		JLoader::register('JevLocationsHelper',JPATH_ADMINISTRATOR."/components/com_jevlocations/libraries/helper.php");
		JevLocationsHelper::loadApiScript();

		$mapsprovider = $compparams->get("mapsprovider", "googlemaps");

		HTMLHelper::script('com_jevents/jevmaps/geolocation.js', array('version' => JeventsHelper::JEvents_Version(false), 'relative' => true));

		$gregion =  JevLocationsHelper::getParameter('googlemaps-service', "gregion", "US");

		$mapquestkey = JevLocationsHelper::getParameter('openstreetmaps-service', "mapquestkey", "");

		$filterList["title"]= "";

		$filterList["html"] =  '<input type="hidden" size="27" name="googleaddress" id="googleaddress" value=""  onchange="clearlonlat();"/>';
		$filterList["html"] .= "<input type='hidden' name='".$this->filterType."_fv' id='".$this->filterType."_fv' class='evlocsearch' value='".$this->filter_value."' />";
		$filterList["html"] .= "<input type='hidden' id='gregion' name='gregion' value='$gregion' />";
		$filterList["html"] .= "<div  id='gaddressmatches' style='display:none;padding:3px;margin:0px!important;min-width:200px;color:#111;background-color:#eee;z-index:999;'  ></div>";


		$scale = " ".$this->params->get("scale","miles");

		$googlesearchdistance= $this->params->get("geosearchdistance",10);

		$filterList["html"] .= "<input type='hidden' id='geosearchdistance' name='geosearchdistance' value='$googlesearchdistance' />";

		$script = "try {JeventsFilters.filters.push({id:'".$this->filterType."_fv',value:''});} catch (e) {}";
		$script .= "try {JeventsFilters.filters.push({id:'googleaddress',value:''});} catch (e) {}";
		// geolocation
		if ($this->params->get("autogeolocate",0)){
			$autogeolocate = $this->params->get("autogeolocate",0);
			$script .= "jQuery(window).load(function(){findEventNearby($autogeolocate);});";
			$filterList["html"] = '<div class="browserlocationfilter" style="display: none;">'.$filterList["html"]."</div>";
		}
		else
		{
			$filterList["html"] .= "<button class='btn-primary' onclick='findEventNearby(0);return false;' >Near Me</button>";
			$filterList["html"] = '<div class="browserlocationfilter" >'.$filterList["html"]."</div>";
		}

		$script .= "var geomapsprovider = '$mapsprovider';";
		$script .= "var mapquestkey = '$mapquestkey';";

		$document = Factory::getDocument();
		$document->addScriptDeclaration($script);

		return $filterList;

	}
}
