<?php
/**
 *
 * @copyright   Copyright (C) 2015 - 2015 GWE Systems Ltd. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

defined('JPATH_PLATFORM') or die;

use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\Document\Document;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;

/**
 * Utility class for JEvents Maps loading
 *
 */
class JEventsJevmapsJevmaps
{
	/**
	 * @var    array  Array containing information for loaded files
	 * @since  3.0
	 */

	protected static $_instance;
	protected static $loaded = array();
	protected $iconsBaseURL = "";
	protected $mapService = 'googlemaps';
	protected $mapId = 'gmapMulti';
	protected $locations = array();
	protected $layoutContext = 'com_jevents';
	protected $mapCenter = 0;
	protected $mapMin = 0;
	protected $mapMax = 0;
	protected $mapUrl = "https://maps.googleapis.com";
	protected $directionsUrl = "https://www.google.com";
	protected $mapZoom = 4;
	protected $mapType = 'roadmap';
	protected $mapStyle = "";
	protected $serviceParams = '';
	protected $mapWidth = '';
	protected $mapHeight = '';
	protected $src = "";

	public function __construct()
	{
		$this->setIconsBaseUrl(Uri::root() . 'media/com_jevents/jevmaps/icons/');
	}

	public static function getInstance()
	{
		if ( !(self::$_instance instanceof self) ) {
            self::$_instance = new self();
        }
        return self::$_instance;
	}

	/**
	 * Method to load the Maps JavaScript framework into the document head
	 *
	 * If debugging mode is on an uncompressed version of Bootstrap is included for easier debugging.
	 *
	 * @param   mixed  $debug  Is debugging mode on? [optional]
	 *
	 * @return  void
	 *
	 * @since   3.0
	 */
	public function framework($mapService='googlemaps',$debug = null)
	{
		// Only load once
		if (!empty(static::$loaded[__METHOD__]))
		{
			return;
		}

		$this->mapService = $mapService;

		switch($this->mapService)
		{
			case "googlemaps":
			default:
				HTMLHelper::script('com_jevents/jevmaps/jevmaps_gmaps.js', true, true, false, false, true);
				$paramsUrl = array();
				if(!empty($this->serviceParams))
				{
					foreach($this->serviceParams as $name => $value)
					{
						$paramsUrl[] = $name . "=" . $value;
					}
				}
				$paramsUrl[] = "callback=JEVENTS.gmaps.init";
				$this->src = $this->mapUrl.'/maps/api/js?'.implode('&',$paramsUrl);

			break;
		}
	
		static::$loaded[__METHOD__] = true;

		return;
	}

	/**
	 * Sets the type of map to show
	 * @param type $type
	 */
	public function setMapType($type)
	{
		$this->mapType = $type;
	}

	public function setMapStyle($style,$custom="")
	{
		switch($style)
		{
			case 'retro':
				$styling = '[ { "elementType": "geometry", "stylers": [ { "color": "#ebe3cd" } ] }, { "elementType": "labels.text.fill", "stylers": [ { "color": "#523735" } ] }, { "elementType": "labels.text.stroke", "stylers": [ { "color": "#f5f1e6" } ] }, { "featureType": "administrative", "elementType": "geometry.stroke", "stylers": [ { "color": "#c9b2a6" } ] }, { "featureType": "administrative.land_parcel", "elementType": "geometry.stroke", "stylers": [ { "color": "#dcd2be" } ] }, { "featureType": "administrative.land_parcel", "elementType": "labels.text.fill", "stylers": [ { "color": "#ae9e90" } ] }, { "featureType": "landscape.natural", "elementType": "geometry", "stylers": [ { "color": "#dfd2ae" } ] }, { "featureType": "poi", "elementType": "geometry", "stylers": [ { "color": "#dfd2ae" } ] }, { "featureType": "poi", "elementType": "labels.text.fill", "stylers": [ { "color": "#93817c" } ] }, { "featureType": "poi.park", "elementType": "geometry.fill", "stylers": [ { "color": "#a5b076" } ] }, { "featureType": "poi.park", "elementType": "labels.text.fill", "stylers": [ { "color": "#447530" } ] }, { "featureType": "road", "elementType": "geometry", "stylers": [ { "color": "#f5f1e6" } ] }, { "featureType": "road.arterial", "elementType": "geometry", "stylers": [ { "color": "#fdfcf8" } ] }, { "featureType": "road.highway", "elementType": "geometry", "stylers": [ { "color": "#f8c967" } ] }, { "featureType": "road.highway", "elementType": "geometry.stroke", "stylers": [ { "color": "#e9bc62" } ] }, { "featureType": "road.highway.controlled_access", "elementType": "geometry", "stylers": [ { "color": "#e98d58" } ] }, { "featureType": "road.highway.controlled_access", "elementType": "geometry.stroke", "stylers": [ { "color": "#db8555" } ] }, { "featureType": "road.local", "elementType": "labels.text.fill", "stylers": [ { "color": "#806b63" } ] }, { "featureType": "transit.line", "elementType": "geometry", "stylers": [ { "color": "#dfd2ae" } ] }, { "featureType": "transit.line", "elementType": "labels.text.fill", "stylers": [ { "color": "#8f7d77" } ] }, { "featureType": "transit.line", "elementType": "labels.text.stroke", "stylers": [ { "color": "#ebe3cd" } ] }, { "featureType": "transit.station", "elementType": "geometry", "stylers": [ { "color": "#dfd2ae" } ] }, { "featureType": "water", "elementType": "geometry.fill", "stylers": [ { "color": "#b9d3c2" } ] }, { "featureType": "water", "elementType": "labels.text.fill", "stylers": [ { "color": "#92998d" } ] } ]';
				break;
			case 'silver':
				$styling = '[{"elementType":"geometry","stylers":[{"color":"#f5f5f5"}]},{"elementType":"labels.icon","stylers":[{"visibility":"off"}]},{"elementType":"labels.text.fill","stylers":[{"color":"#616161"}]},{"elementType":"labels.text.stroke","stylers":[{"color":"#f5f5f5"}]},{"featureType":"administrative.land_parcel","elementType":"labels.text.fill","stylers":[{"color":"#bdbdbd"}]},{"featureType":"poi","elementType":"geometry","stylers":[{"color":"#eeeeee"}]},{"featureType":"poi","elementType":"labels.text.fill","stylers":[{"color":"#757575"}]},{"featureType":"poi.park","elementType":"geometry","stylers":[{"color":"#e5e5e5"}]},{"featureType":"poi.park","elementType":"labels.text.fill","stylers":[{"color":"#9e9e9e"}]},{"featureType":"road","elementType":"geometry","stylers":[{"color":"#ffffff"}]},{"featureType":"road.arterial","elementType":"labels.text.fill","stylers":[{"color":"#757575"}]},{"featureType":"road.highway","elementType":"geometry","stylers":[{"color":"#dadada"}]},{"featureType":"road.highway","elementType":"labels.text.fill","stylers":[{"color":"#616161"}]},{"featureType":"road.local","elementType":"labels.text.fill","stylers":[{"color":"#9e9e9e"}]},{"featureType":"transit.line","elementType":"geometry","stylers":[{"color":"#e5e5e5"}]},{"featureType":"transit.station","elementType":"geometry","stylers":[{"color":"#eeeeee"}]},{"featureType":"water","elementType":"geometry","stylers":[{"color":"#c9c9c9"}]},{"featureType":"water","elementType":"labels.text.fill","stylers":[{"color":"#9e9e9e"}]}]';
				break;
			case 'night':
				$styling = '[{"elementType":"geometry","stylers":[{"color":"#242f3e"}]},{"elementType":"labels.text.fill","stylers":[{"color":"#746855"}]},{"elementType":"labels.text.stroke","stylers":[{"color":"#242f3e"}]},{"featureType":"administrative.locality","elementType":"labels.text.fill","stylers":[{"color":"#d59563"}]},{"featureType":"poi","elementType":"labels.text.fill","stylers":[{"color":"#d59563"}]},{"featureType":"poi.park","elementType":"geometry","stylers":[{"color":"#263c3f"}]},{"featureType":"poi.park","elementType":"labels.text.fill","stylers":[{"color":"#6b9a76"}]},{"featureType":"road","elementType":"geometry","stylers":[{"color":"#38414e"}]},{"featureType":"road","elementType":"geometry.stroke","stylers":[{"color":"#212a37"}]},{"featureType":"road","elementType":"labels.text.fill","stylers":[{"color":"#9ca5b3"}]},{"featureType":"road.highway","elementType":"geometry","stylers":[{"color":"#746855"}]},{"featureType":"road.highway","elementType":"geometry.stroke","stylers":[{"color":"#1f2835"}]},{"featureType":"road.highway","elementType":"labels.text.fill","stylers":[{"color":"#f3d19c"}]},{"featureType":"transit","elementType":"geometry","stylers":[{"color":"#2f3948"}]},{"featureType":"transit.station","elementType":"labels.text.fill","stylers":[{"color":"#d59563"}]},{"featureType":"water","elementType":"geometry","stylers":[{"color":"#17263c"}]},{"featureType":"water","elementType":"labels.text.fill","stylers":[{"color":"#515c6d"}]},{"featureType":"water","elementType":"labels.text.stroke","stylers":[{"color":"#17263c"}]}]';
				break;
			case 'custom':
				$styling = $custom;
				break;
			default:
				$styling = '';
		}

		$this->mapStyle = $styling;
	}

	public function setIconsBaseUrl($url)
	{
		if($url)
		{
			$this->iconsBaseURL = $url;
		}
	}

	/**
	 * Adds points to be shown on the map
	 * @param type $latitude
	 * @param type $longitude
	 * @param type $locid
	 * @param type $title
	 * @param type $url
	 * @param type $evttitle
	 * @param type $icon
	 * @param string $infoLayout
	 */
	public function addLocation($latitude, $longitude, $locid, $title, $url, $evttitle, $icon,$zoom=0, $infoLayout = '', $mapId='gmap')
	{
		$location = new stdClass();

		$location->latitude = $latitude;
		$location->longitude = $longitude;
		$location->locid = $locid;
		$location->title = $title;
		$location->url = $url;
		$location->evttitle = $evttitle;
		$location->icon = $icon;
		$location->zoom = $zoom;
		$location->infoLayout = $infoLayout;
		$location->mapId = $mapId;

		$this->locations[] = $location;
	}

	/**
	 * This methods puts everything together to load the map
	 */
	public function loadMap( $layoutPath = 'people.map')
	{
		$script = array();

		$options = new stdClass();
		$options->center = json_encode($this->getMapCenter());
		$options->mapType = $this->mapType;
		$options->zoom = $this->mapZoom;
		
		$script[] = 'JEVENTS.gmaps.setIconsBaseUrl("' . $this->iconsBaseURL .'");';
		if($this->mapStyle!= '')
		{
			$script[] = 'JEVENTS.gmaps.setMapStyle(' . $this->mapStyle . ');';
		}
		$script[] = 'JEVENTS.gmaps.setOptions(' .  json_encode($options). ');';
		$script[] = 'JEVENTS.gmaps.setMapId("' . $this->mapId .'");';		

		foreach($this->locations as $location)
		{
			$script[] = 'JEVENTS.gmaps.addLocation(' .  json_encode($location). ');';
		}

		$script[] = 'JEVENTS.gmaps.setBounds(' . json_encode($this->mapMin) . ', '. json_encode($this->mapMax) .');';

		$this->addJavaScript($script);

		$displayData = array('mapId'=>$this->mapId, 'mapHeight'=>$this->mapHeight, 'mapWidth'=>$this->mapWidth);
		$layoutOptions['component'] = $this->layoutContext;

		$html = LayoutHelper::render( $layoutPath, $displayData, null, 
						array('debug'=>false,'client'=>0),
						$layoutOptions );
		$document = Factory::getDocument();
		$document->addScript( $this->src, array(), array( 'async'=>'async', 'defer'=>'defer' ) );

		return $html;
	}	

	private function getMapCenter()
	{
		if($this->mapCenter === 0)
		{
			$minlon = 0;
			$minlat = 0;
			$maxlon = 0;
			$maxlat = 0;
			$first = true;
			foreach ($this->locations as $location)
			{
				if ($location->zoom == 0)
					continue;
				if ($first)
				{
					$minlon = floatval($location->longitude);
					$minlat = floatval($location->latitude);
					$maxlon = floatval($location->longitude);
					$maxlat = floatval($location->latitude);
					$first = false;
				}
				$minlon = floatval($location->longitude) > $minlon ? $minlon : floatval($location->longitude);
				$minlat = floatval($location->latitude) > $minlat ? $minlat : floatval($location->latitude);
				$maxlon = floatval($location->longitude) < $maxlon ? $maxlon : floatval($location->longitude);
				$maxlat = floatval($location->latitude) < $maxlat ? $maxlat : floatval($location->latitude);
			}
			if ($minlon == $maxlon)
			{
				$minlon-=0.002;
				$maxlon+=0.002;
			}
			if ($minlat == $maxlat)
			{
				$minlat-=0.002;
				$maxlat+=0.002;
			}

			$this->mapCenter = new stdClass();
			$this->mapMin = new stdClass();
			$this->mapMax = new stdClass();
			$this->mapCenter->lng = ($minlon + $maxlon) / 2.0;
			$this->mapCenter->lat = ($minlat + $maxlat) / 2.0;
			$this->mapMin->lat = $minlat;
			$this->mapMin->lng = $minlon;
			$this->mapMax->lat = $maxlat;
			$this->mapMax->lng = $maxlon;
		}

		return $this->mapCenter;
	}

	/**
	 * This method gets an script with all javascript lines to add to the page and injects them into Document
	 * @param array $script
	 */
	private function addJavaScript($script=array())
	{
		$doc = Factory::getDocument();

		$doc->addScriptDeclaration(implode('',$script));
	}	
	
	public function setLayoutContext($context)
	{
		$this->layoutContext = $context;
	}

	public function setMapUrl($url)
	{
		$this->mapUrl = $url;
	}

	public function setMapHeight($height = "300px")
	{
		$this->mapHeight = $height;
	}

	public function setMapWidth($width = "100%")
	{
		$this->mapWidth = $width;
	}

	public function setDirectionsUrl($url)
	{
		$this->directionsUrl = $url;
	}

	public function setMapId($id)
	{
		$this->mapId = $id;
	}

	public function setServiceParams($params)
	{
		$this->serviceParams = $params;
	}

	public function getDirectionsUrl($title, $geolat, $geolon, $geozoom=10)
	{
		$url = $this->directionsUrl . "/maps?f=q&geocode=&time=&date=&ttype=&ie=UTF8&t=h&om=1&q=" . $title . "@" . $geolat . "," . $geolon . "&ll=" . $geolat . "," . $geolon . "&z=" . $geozoom . "&iwloc=addr";

		return $url;
	}
	
}
